#include "global.h"

#if defined (_WINDOWS)
	#include <direct.h>
	#include <windows.h>
#elif defined (__unix__)
	#include <sys/stat.h>
#endif
#include <string>
#include <iostream>
#include <time.h>		// om time stamps te maken, en wellicht RandomUtils te inniten
#include <omp.h>

// -- qtils
#include <RandomUtils.h>
#include <Config.h>
#include <FileUtils.h>
#include <StringUtils.h>
#include <SystemUtils.h>
#include <TimeUtils.h>
#include <logger/Log.h>

// -- bass
#include <Bass.h>
#include <db/DbFile.h>

#include "FicConf.h"
#include "tasks/TaskHandler.h"
#include "tasks/MainTH.h"

#include "FicMain.h"

const string FicMain::ficVersion = "SSGMiner v1";

FicMain::FicMain(Config *config) {
	mConfig = config;
	mTaskClass = mConfig->Read<string>("taskclass");

	Bass::ParseGlobalSettings(mConfig);
}
FicMain::~FicMain() {
	
}

/* ------------------------- Main execution -------------------------- */

void FicMain::Execute() {
	// Init TaskHandler
	TaskHandler *th = NULL;
			if(mTaskClass.compare("main") == 0)				th = new MainTH(mConfig);
	else	throw "No TaskHandler available for `" + mTaskClass + "`";

	// Init command in AppLog
	if(mConfig->Read<uint32>("logstuff",1) == 1)
		Logger::CommandInit(mTaskClass, mConfig->Read<string>("command"));

	// Set working directory, based on TaskHandler
	string wd = Bass::GetExpDir() + th->BuildWorkingDir();
	Bass::SetWorkingDir(wd);

	// Write config file to working dir
	string confFileName = Bass::GetWorkingDir() + mConfig->Read<string>("command") + "-" + Logger::GetStartTimeString() + ".conf";
	mConfig->WriteFile(confFileName);

	// Configure Logger
	Logger::SetAppLogDir(Bass::GetExecDir());
	Logger::SetCommandLogDir(Bass::GetWorkingDir());
	Logger::Init(th->ShouldWriteTaskLogs());

	// Some old stuff that needs to be changed
	Bass::SetOutputLevel(mConfig->Read<uint32>("verbosity", 2));
	ECHO(1,printf(" * Verbosity:\t\t%d\n", Bass::GetOutputLevel()));
	//ECHO(1, printf(" * Max Mem Usage:\t%dmb\n",(uint32)(Bass::GetMaxMemUse()/(uint64)MEGABYTE)));

	if(mConfig->KeyExists("takeiteasy") && mConfig->Read<uint32>("takeiteasy") != 0) {
		ECHO(1, printf(Bass::GetOutputLevel() > 1 ? " * Priority:\t\tYa man, i and i gonna take it easy, man...\n" : " * Priority:\t\tLow\n"))
			SystemUtils::TakeItEasy();
	} else
		ECHO(1, printf(Bass::GetOutputLevel() > 1 ? " * Priority:\t\tOpzij, opzij, opzij!\n\t\t\tMaak plaats, maak plaats, maak plaats!\n\t\t\tWij hebben ongelovelijke haast!\n" : " * Priority:\t\tHigh\n"))
	if(Bass::GetNumThreads() > 1)
		ECHO(1,printf(" * Threaded:\t\tusing %d threads\n", Bass::GetNumThreads()));

	ECHO(1,printf("\n"));

	// Execute task at hand
	th->HandleTask();	// throws error if it can't handle it, baby.

	// Finished
	ECHO(1,printf("\n * Check results in:\t%s\n", Bass::GetWorkingDir().c_str()));

	// Round up command.log
	if(mConfig->Read<uint32>("logstuff",1) == 1)
		Logger::CommandEnd();
	Logger::CleanUp();

	// Cleanup
	delete th;
}
