#ifndef __PATTERNCOLLECTION_H
#define __PATTERNCOLLECTION_H

#include "../BassApi.h"
#include "../itemstructs/ItemSet.h"

class Database;
class PatternFile;

class BASS_API PatternCollection {
public:
	PatternCollection(Database *db, PatternFile *pFile);
	virtual ~PatternCollection();

	// Modify collection
	virtual void		Add(ItemSet *is) =0;
	virtual void		NextIteration() {}
	// At the end of the search, give the collection the opportunity to e.g. save
	virtual void		Finalize() {}

	virtual double		GetMinThreshold() { return DOUBLE_MIN_VALUE; }
	virtual uint32		GetNumIterations() { return 1; }
	uint64				GetNumPatterns() { return mNumPatterns; }
	virtual ItemSet		**GetPatterns()   { THROW_NOT_IMPLEMENTED(); }
	virtual ItemSet		*GetPattern(const uint32 num)   { THROW_NOT_IMPLEMENTED(); }

	// Compute more statistics
	virtual void		WritePatternDetails(const string &filename)	{ THROW_NOT_IMPLEMENTED(); }
	virtual void		ComputeGlobalStats()			{ THROW_NOT_IMPLEMENTED(); }
	double				GetMaxInterest() { return mMaxInterest; }
	double				GetAvgInterest() { return mAvgInterest; }
	double				GetMinInterest() { return mMinInterest; }
	uint32				GetMaxSize() { return mMaxSize; }
	double				GetAvgSize() { return mAvgSize; }
	uint32				GetMinSize() { return mMinSize; }
	double				GetMaxJaccard() { return mMaxJaccard; }
	double				GetAvgJaccard() { return mAvgJaccard; }
	double				GetMinJaccard() { return mMinJaccard; }
	uint32				GetCoverage() { return mCoverage; }

	// Check properties of instances
	virtual bool		IsLoaded() =0;
	
	// Factory
	static PatternCollection* Create(Config *config, Database *db, PatternFile *pFile);

protected:
	Database	*mDatabase;
	PatternFile *mPatternFile;

	uint64		mNumPatterns;

	// Stats
	double		mMaxInterest;
	double		mAvgInterest;
	double		mMinInterest;
	uint32		mMaxSize;
	double		mAvgSize;
	uint32		mMinSize;
	double		mMaxJaccard;
	double		mAvgJaccard;
	double		mMinJaccard;
	uint32		mCoverage;
};
#endif // __PATTERNCOLLECTION_H
