#include "../Bass.h"

#include "Database.h"
#include "../itemstructs/Uint32ItemSet.h"

#include "UnoDbFile.h"

UnoDbFile::UnoDbFile() {

}
UnoDbFile::~UnoDbFile() {

}

Database *UnoDbFile::Read(string filename) {
	ECHO(2, printf("** Database :: \n"));
	ECHO(2,printf(" * Type:\t\tUNO graph database\n"));
	uint32 pos = (uint32) filename.find_last_of("\\/");
	ECHO(2, printf(" * File:\t\t%s\n", pos==string::npos ? (filename.length()>55 ?	filename.substr(filename.length()-55,filename.length()-1).c_str() : filename.c_str()) : filename.substr(pos+1,filename.length()-1).c_str()));
	FILE *fp = NULL;
	char *bp = NULL, *buffer = new char[UNODBFILE_BUFFER_LENGTH + 1];
	if(fopen_s(&fp, filename.c_str(), "r") != 0)
		throw "UnoDbFile::Read() --- Could not read file.";

	uint32 numRows = 0;
	uint32 maxCols = 0;
	uint32 colCount = 0;
	ItemSet **data = NULL;
	uint32 *rowLengths = NULL;
	uint64 transactionId = 0; 

	if(fp != NULL) {
		ECHO(2,printf(" * Scanning file:\tin progress..."));
		if(fgets(buffer, UNODBFILE_BUFFER_LENGTH, fp) == NULL || strlen(buffer) == 0) {
			delete[] buffer;
			throw string("Error while scanning DB (at first line).");
		}
		size_t bufferLen = strlen(buffer);
		if(bufferLen == UNODBFILE_BUFFER_LENGTH-1)
			throw string("UnoDbFile::Read() -- Line does not fit in buffer.");
	
		maxCols = this->CountCols(buffer);
		do {
			colCount = this->CountCols(buffer);
			numRows++;
			if(colCount > maxCols)
				maxCols = colCount;
		} while(fgets(buffer, UNODBFILE_BUFFER_LENGTH, fp) != NULL);
		
		ECHO(2,printf("\r * Scanning file: done                       \n"));
		ECHO(2,printf(" * Database:\t%d nodes, maximum %d edges per node\n", numRows, maxCols));
		if(ferror(fp)) {
			delete[] buffer;
			throw "Error while scanning DB.";
		}
		rewind(fp);

		data = new ItemSet *[numRows];
		for(uint32 i=0; i<numRows; i++)
			data[i] = ItemSet::CreateEmpty(mPreferredDataType, numRows); //numRows == alphabetSize!
		uint32 row = 0;
		uint32 rowLen = 0;

		ECHO(2,printf(" * Reading database:\tin progress...\r"));
		while(fgets(buffer, UNODBFILE_BUFFER_LENGTH, fp) != NULL) {
			if(strlen(buffer) > 0 && buffer[0] != '\n') {
				rowLen = this->CountCols(buffer);
				bp = buffer;
				for(uint32 val=0; val<rowLen; val++) {
					uint32 col = atoi(bp);
					data[row]->AddItemToSet(col);
					data[col]->AddItemToSet(row);
					while(*bp != ',' && *bp != '\n')
						bp++;
					bp++;
				}
			}
			row++;
		}
		if(ferror(fp))
			throw "UnoDbFile::Read() --- Error while reading DB.";
		fclose(fp);
		ECHO(2,printf(" * Reading database:\tdone          \n"));
	} else {	// File open error!
		delete[] buffer;
		throw "UnoDbFile::Read() --- Could not read database.";
	}

	// Check if there is a `types' file -- if yes, read
	uint32 numEntityTypes = 0;
	string typesname = filename.substr(0, filename.length()-3) + "types"; // - uno + types
	if(FileUtils::FileExists(typesname)) {
		// Yes, it's there
		if(fopen_s(&fp, typesname.c_str(), "r") != 0)
			throw "UnoDbFile::Read() --- Could not read file.";

		if(fp != NULL) {
			if(fgets(buffer, UNODBFILE_BUFFER_LENGTH, fp) == NULL || strlen(buffer) == 0) {
				delete[] buffer;
				throw string("Error while scanning DB (at first line).");
			}
			numEntityTypes = atoi(buffer);

			uint32 row = 0;
			while(fgets(buffer, UNODBFILE_BUFFER_LENGTH, fp) != NULL) {
				if(strlen(buffer) > 0 && buffer[0] != '\n') {
					//data[row]->SetEntityType(atoi(buffer));
					row++;
				}
			}
			if(row != numRows)
				throw "UnoDbFile::Read() --- Did not read numRows rows.";
			if(ferror(fp))
				throw "UnoDbFile::Read() --- Error while reading types file.";
			fclose(fp);
		} else {
			delete[] buffer;
			throw "UnoDbFile::Read() --- Could not read types file.";
		}
	}

	// Continue
	delete[] buffer;

	Database *db = new Database(data, numRows, numEntityTypes);
	db->ComputeEssentials();
	ECHO(2,printf(" * Total number of edges: %I64d.\n", db->GetNumItems()));

	// Path & filename
	string path, ext;
	Database::DeterminePathFileExt(path, filename, ext, DbFileTypeUno);
	db->SetPath(path);
	db->SetFilename(filename);

	ECHO(3,printf("** Finished reading database.\n"));
	ECHO(2,printf("\n"));
	return db;
}

uint16 UnoDbFile::CountCols(char *buffer) {
	uint32 count = 0, index = 0, find = 0;
	while(buffer[index] != '\n')
		if(buffer[index++] == ',')
			count++;
	if(buffer[index] == '\n' && index > 0 && buffer[index-1] != ',')
		count++;
	return count;
}

bool UnoDbFile::Write(Database *db, const string &fullpath) {
	printf("Cannot write UNO database.\n");
	return false;
}
