#include "../blobal.h"

#include <RandomUtils.h>

#include "../Bass.h"
#include "../db/Database.h"

//#include "Uint16ItemSet.h"
//#include "BAI32ItemSet.h"
//#include "BM128ItemSet.h"
#include "Uint32ItemSet.h"
#include "ItemSet.h"

ItemSet::ItemSet() {
	mNumEdges = 0;
	mEdgeProbability = 0.0;
	mLambda = DOUBLE_MIN_VALUE;
	mUpperBound = DOUBLE_MAX_VALUE;
	mId = 0;
}

ItemSet::ItemSet(const ItemSet& is) {
	mNumEdges = is.mNumEdges;
	mEdgeProbability = is.mEdgeProbability;
	mLambda = is.mLambda;
	mUpperBound = is.mUpperBound;
	mId = is.mId;
}

ItemSet::~ItemSet() {
	
}

// set wordt netjes gekopieerd, ownership wordt dus niet overgenomen.
ItemSet *ItemSet::Create(ItemSetType type, const uint32 *set, uint32 setlen, uint32 abetlen, uint32 numEdges, double lambda) {
	if(type == Uint32ItemSetType)
		return new Uint32ItemSet(set, setlen, numEdges, lambda);
	throw "Wrong ItemSetType.";
}
ItemSet *ItemSet::CreateSingleton(ItemSetType type, uint32 elem, uint32 abetlen, uint32 numEdges, double lambda) {
	if(type == Uint32ItemSetType)
		return new Uint32ItemSet(elem, numEdges, lambda);
	throw "Wrong ItemSetType.";
}
ItemSet *ItemSet::CreateRange(ItemSetType type, uint32 from, uint32 to, uint32 abetlen, uint32 numEdges, double lambda) {
	if(type == Uint32ItemSetType)
		return new Uint32ItemSet(from, to, numEdges, lambda);
	throw "Wrong ItemSetType.";
}
ItemSet *ItemSet::CreateEmpty(ItemSetType type, size_t abetlen) {
	if(type == Uint32ItemSetType)
		return new Uint32ItemSet();
	throw "Wrong ItemSetType.";
}

uint32 ItemSet::MemUsage(const ItemSetType type, const uint32 abLen, const uint32 setLen) {
	if(type == Uint32ItemSetType)
		return Uint32ItemSet::MemUsage(setLen);
	throw "Wrong ItemSetType.";
}
uint32 ItemSet::MemUsage(Database *db) {
	ItemSetType type = db->GetDataType();
	if(type == Uint32ItemSetType)
		return Uint32ItemSet::MemUsage(db->GetMaxSetLength());
	//else if(type == BAI32ItemSetType)
	//	return BAI32ItemSet::MemUsage((uint32)db->GetAlphabetSize());
	//else if(type == Uint16ItemSetType)
	//	return Uint16ItemSet::MemUsage(db->GetMaxSetLength());
	//else if(type == BM128ItemSetType)
	//	return BM128ItemSet::MemUsage();
	throw "Wrong ItemSetType.";
}

ItemSetType ItemSet::StringToType(const string desc) {
	if(desc.compare("uint32") == 0)
		return Uint32ItemSetType;
	//else if(desc.compare("uint16") == 0)
	//	return Uint16ItemSetType;
	//else if(desc.compare("bai32") == 0)
	//	return BAI32ItemSetType;
	//else if(desc.compare("bm128") == 0)
	//	return BM128ItemSetType;
	return NoneItemSetType;
}
string ItemSet::TypeToString(const ItemSetType type) {
	if(type == Uint32ItemSetType)
		return "uint32";
	//else if(type == BAI32ItemSetType)
	//	return "bai32";
	//else if(type == Uint16ItemSetType)
	//	return "uint16";
	//else if(type == BM128ItemSetType)
	//	return "bm128";
	return "ufo";
}

string ItemSet::TypeToFullName(const ItemSetType type) {
	if(type == Uint32ItemSetType)
		return "uint32 value arrays";
	//else if(type == BAI32ItemSetType)
	//	return "32bit bitmap array";
	//else if(type == Uint16ItemSetType)
	//	return "uint16 value arrays";
	//else if(type == BM128ItemSetType)
	//	return "128bit bitmap";
	return "aliens";
}
ItemSet* ItemSet::Remainder(ItemSet *is) const {
	ItemSet *retval = this->Clone();
	retval->Remove(is);
	return retval;
}

