#include "../Bass.h"
#include "../db/Database.h"

#include <sys/stat.h>
#include <FileUtils.h>

#include "PatternFile.h"

PatternFile::PatternFile(const string filename, Database *db) {
	mFile = NULL;
	mFilename = filename;
	mDatabase = db;

	mSetArray = new uint32[mDatabase->GetNumRows()];

	OpenFile(filename, FileWritable);
}
PatternFile::~PatternFile() {
	CloseFile();
	delete[] mSetArray;
}
void PatternFile::OpenFile(const string &filename, const FileOpenType openType /* = FileReadable */) {
	if(mFile != NULL)
		THROW("Filehandle already open");

	if(fopen_s(&mFile, filename.c_str(), FileUtils::FileOpenTypeToString(openType).c_str()))
		THROW("Unable to open file");

	mFilename = filename;
}
void PatternFile::CloseFile() {
	if(mFile != NULL)
		fclose(mFile);
	mFile = NULL;
	mFilename = "";
}
bool PatternFile::IsOpen() {
	return mFile != NULL;
}
void PatternFile::WriteItemSet(ItemSet * is) {
	if(mFile == NULL)
		THROW("No file handle open");
	if(ferror(mFile))
		THROW("Error writing pattern");

	uint32 len = is->GetLength();
	is->GetValuesIn(mSetArray);

	char buffer[ISCFILE_BUFFER_LENGTH];
	char *bp = buffer;

	for(uint32 j=0; j<len; j++) {
		_itoa_s(mSetArray[j], bp, ISCFILE_BUFFER_LENGTH - (bp - buffer), 10);
		bp += strlen(bp);
		*bp = ',';
		bp++;
	}
	if(len > 0)
		*(bp-1) = ' ';

	*bp = '(';
	bp++;
	_itoa_s(is->GetLength(), bp, ISCFILE_BUFFER_LENGTH - (bp - buffer), 10);
	bp += strlen(bp);
	*bp = ',';
	bp++;
	_itoa_s(is->GetNumEdges(), bp, ISCFILE_BUFFER_LENGTH - (bp - buffer), 10);
	bp += strlen(bp);
	*bp = ',';
	bp++;
	sprintf_s(bp, ISCFILE_BUFFER_LENGTH - (bp - buffer), "%f", is->GetLambda());
	bp += strlen(bp);
	*bp = ')';
	*(bp+1) = '\n';
	bp+=2;

	fwrite(buffer,sizeof(char),bp - buffer,mFile);
}
