#ifndef __UINT32ITEMSET_H
#define __UINT32ITEMSET_H

#include "ItemSet.h"

class BASS_API Uint32ItemSet : public ItemSet {
public:
	// Copy* Constructor
	Uint32ItemSet(const Uint32ItemSet* iset);
	// Create empty Uint32ItemSet
	Uint32ItemSet();
	// Create singleton Uint32ItemSet
	Uint32ItemSet(uint32 elem, uint32 numEdges, double lambda=0);
	// Create range Uint32ItemSet
	Uint32ItemSet(uint32 from, uint32 to, uint32 numEdges, double lambda=0);
	// Create Uint32ItemSet
	Uint32ItemSet(const uint32 *set, uint32 setlen, uint32 numEdges=0, double lambda=0);

	virtual ~Uint32ItemSet();
	
	virtual Uint32ItemSet*	Clone() const;

	virtual uint32		GetLength() { return mLength; }

	// Memory-Usage estimator for this Uint32ItemSet
	virtual uint32		GetMemUsage() const ;
	// Generic Memory-Usage estimator for an Uint32ItemSet for a set of length `setLen`
	static uint32		MemUsage(const uint32 setlen);

	// Enumerator should be used in conjunction with for-loop over Length
	//virtual void		ResetEnumerator();
	//virtual uint32		GetNext();

	virtual void		RemoveDoubleItems(); // Uint32 only!

	virtual void		AddItemToSet(uint32 val);
	virtual void		RemoveItemFromSet(uint32 val);
	virtual bool		IsItemInSet(uint32 val) const;
	// Returns the last item in the set. Only call if length>0, as we do not check that here.
	virtual const uint32	GetLastItem() const { return (mLength > 0 ? mSet[mLength-1] : UINT32_MAX_VALUE); };

	// Fills the set with 0's from [o,max>
	virtual void		FillHerUp(uint32 max);
	virtual void		CleanTheSlate();

	virtual bool		Equals(ItemSet *is) const;
	virtual bool		Intersects(ItemSet *is) const;
	virtual uint32		IntersectionSize(ItemSet *is) const;
	virtual bool		IsSubset(ItemSet *is) const;

	virtual float		JaccardDistance(ItemSet *is) const;

	virtual uint32		CountItemsGEQThan(const uint32 n) const;

	virtual void		Remove(ItemSet *is);

	virtual ItemSet*	Union(ItemSet *is) const;
	virtual ItemSet*	Intersection(ItemSet *is) const;

	virtual void		Unite(ItemSet *is);

	virtual void		Sort();

	// returns ownership of an uint32 array with therein the values of the item set
	virtual uint32*		GetValues() const;
	// returns values of the item set in the provided uint32 values array
	virtual void		GetValuesIn(uint32* vals) const;
	// increment the values of the item set in the provided uint32 values array
	virtual void		IncrementValues(uint32* vals, int32 cnt) const;

	//////////////// SUBGRAPH-SPECIFIC METHODS /////////////////

	virtual SubgraphStats ComputeSubgraphStats(Database *db);

	//// Rest ////

	virtual ItemSetType GetType() const { return Uint32ItemSetType; }

	// needed by Uint32CoverSet
	virtual uint32*		GetSet() const { return mSet; }
	void				SetSet(uint32* set, uint32 len) { mSet = set; mLength = len; }

	virtual string		ToString(bool printNums=true, bool printLambda=false);

	// Compare item sets on alphabetical order. Returns 0 iff equal, -1 iff this<is, 1 iff this>is.
	virtual int8		CmpLexicographically(ItemSet *b);

protected:
	void				SetLength(uint32 length) { mLength = length; }

	void				QSortAscending(int first, int last);
	__inline void		SwapItems(int a, int b) { uint32 t = mSet[a]; mSet[a] = mSet[b]; mSet[b] = t; }

	uint32	*mSet;
	uint32	mLength;

	friend class Uint32CoverSet;
};

#endif // __UINT32ITEMSET_H
