#include "../global.h"

#include <Bass.h>
#include <Config.h>
#include <logger/Log.h>

#include <db/Database.h>

#include "MaxEntInterestingness.h"
#include "UniformInterestingness.h"

#include "Interestingness.h"

Interestingness::Interestingness(Config *config, Database *db) {
	mConfig	  = config;
	mDatabase = db;
	mNumNodes = db->GetNumRows();

	// For description length
	mDescriptionQ		= config->Read<double>("descriptionQ");
	if(mDescriptionQ <= 0.0 || mDescriptionQ >= 1.0)
		THROW("descriptionQ should be a probability, 0 < q < 1.");
	mDescIncludeCost = Log2((1.0 - mDescriptionQ) / mDescriptionQ);
	mDescExcludeCost = mNumNodes * Log2(1.0 / (1.0 - mDescriptionQ));

	// Compute theoretical max#edges for all possible subgraph sizes (used for computing edge densities)
	mMaxNumEdges = new uint32[mNumNodes+1];
	for(uint32 n=0; n<=mNumNodes; n++)
		mMaxNumEdges[n] = n * (n-1) / 2;
}
Interestingness::~Interestingness() {
	delete[] mMaxNumEdges;
}
void Interestingness::LogParameters() {
	LOG_MSG("Interestingness:\t" + GetName());
	LOG_MSG("description q:\t\t" + mConfig->Read<string>("descriptionQ"));
}
Interestingness* Interestingness::Create(Config *config, Database *db) {
	string desc = config->Read<string>("interestingness");
	if(desc.compare("maxent") == 0)
		return new MaxEntInterestingness(config, db);
	if(desc.compare("uniform") == 0)
		return new UniformInterestingness(config, db);
	THROW("Unknown interestingness type.");
}
