#ifndef __INTERESTINGNESS_H
#define __INTERESTINGNESS_H

#include <Bass.h>

class Config;
class Database;
class ItemSet;

class Interestingness {
public:
	Interestingness(Config *config, Database *db);
	virtual ~Interestingness();

	// Complete interestingness computation for given pattern
	virtual void		ForPattern(ItemSet *pattern) =0;
	// Compute quality (and everything required for this) for a newly extended pattern and store this in the pattern (pattern = parent + node)
	virtual void		ForExtension(ItemSet *pattern, ItemSet *parent, const uint32 node, const uint32 newNumNodes) =0;
	// Compute quality (and everything required for this) for a newly 'reduced' pattern and store this in the pattern (pattern = parent \ node)
	virtual void		ForReduction(ItemSet *pattern, ItemSet *parent, const uint32 node, const uint32 newNumNodes) =0;

	//virtual bool		CanSingleExtensionImprove(ItemSet *pattern, const uint32 fromNode, const uint32 toNode, const uint32 newNumNodes) =0;

	// Update interestingness by adding given pattern to background knowledge, for iterative mining
	virtual void		AddToBackground(ItemSet *pattern) =0;

	virtual string		GetName() =0;
	virtual void		LogParameters();
	virtual bool		HasUpperBound()							{ return false; }
	virtual void		ComputeUpperBound(ItemSet *pattern, const uint32 fromNode)		{}

	// ---- Static ----
	static Interestingness* Create(Config *config, Database *db);

protected:

	// Parameters
	double		mDescriptionQ;		// q in DescriptionLength

	// Member variables
	Config		*mConfig;
	Database	*mDatabase;
	uint32		mNumNodes;			// number of nodes in complete graph
	uint32		*mMaxNumEdges;		// maximum number of edges possible for a given graph size (assuming regular undirected graph, not e.g. multi-relational)

	double		mDescIncludeCost;	// cost to assimilate inclusion of a node in a pattern
	double		mDescExcludeCost;	// cost to assimilate exclusion of a node from a pattern
};

#endif // __INTERESTINGNESS_H
