#ifndef __MAXENTINTERESTINGNESS_H
#define __MAXENTINTERESTINGNESS_H

class MaxEntModel;

#include "Interestingness.h"

class MaxEntInterestingness : public Interestingness {
public:
	MaxEntInterestingness(Config *config, Database *db);
	virtual ~MaxEntInterestingness();

	// Complete interestingness computation for given pattern
	virtual void		ForPattern(ItemSet *pattern);
	// Compute quality (and everything required for this) for a newly extended pattern and store this in the pattern (pattern = parent + node)
	virtual void		ForExtension(ItemSet *pattern, ItemSet *parent, const uint32 node, const uint32 newNumNodes);
	// Compute quality (and everything required for this) for a newly 'reduced' pattern and store this in the pattern (pattern = parent \ node)
	virtual void		ForReduction(ItemSet *pattern, ItemSet *parent, const uint32 node, const uint32 newNumNodes);

	// Update interestingness by adding given pattern to background knowledge, for iterative mining (ie update background distribution)
	virtual void		AddToBackground(ItemSet *pattern);

	virtual string		GetName()				{ return string("degree-based MaxEnt distribution"); }
	virtual void		LogParameters();
	virtual bool		HasUpperBound()			{ return false; }

	MaxEntModel			*GetModel()				{ return mModel;  }

protected:
	// Member variables
	MaxEntModel *mModel;
};

#endif // __MAXENTINTERESTINGNESS_H
