#ifndef __UNIFORMINTERESTINGNESS_H
#define __UNIFORMINTERESTINGNESS_H

#include "Interestingness.h"

class UniformInterestingness : public Interestingness {
public:
	UniformInterestingness(Config *config, Database *db);
	virtual ~UniformInterestingness();

	// Complete interestingness computation for given pattern
	virtual void		ForPattern(ItemSet *pattern);
	// Compute and set interestingness for given extension
	virtual void		ForExtension(ItemSet *pattern, ItemSet *parent, const uint32 node, const uint32 newNumNodes);
	// Compute quality (and everything required for this) for a newly 'reduced' pattern and store this in the pattern (pattern = parent \ node)
	virtual void		ForReduction(ItemSet *pattern, ItemSet *parent, const uint32 node, const uint32 newNumNodes);

	// Update interestingness by adding given pattern to background knowledge, for iterative mining
	virtual void		AddToBackground(ItemSet *pattern);

	virtual string		GetName()			{ return string("uniform distribution"); }
	virtual void		LogParameters();
	virtual bool		HasUpperBound()							{ return true; }
	virtual void		ComputeUpperBound(ItemSet *pattern, const uint32 fromNode);

protected:
	inline double		GetItFor(const uint32 numEdges, const uint32 numNodes) {
		if(numNodes < mCacheUpTo) {
			return mCache[numNodes][numEdges];
		} else {
			return Compute(numEdges, numNodes);
		}
	}
	inline double		Compute(const uint32 numEdges, const uint32 numNodes) {
		if(numNodes==1) return 0.0;
		return (mMaxNumEdges[numNodes] * KL((double)numEdges / mMaxNumEdges[numNodes], mAlpha)) / 
							(numNodes * mDescIncludeCost + mDescExcludeCost);
	}

	// Parameters
	double		mAlpha;				// expected edge density/probability (based on full graph, or as expected by user)
	
	// Member variables
	uint32		mNumNodesUsed; // <= mNumNodes, which is the total number of nodes in the network before removing any.
	uint32		mNumDegreeCounts;
	uint32		*mUBDegreeCounts;
	uint32		*mUBDegreeCountsZeroes;
	uint32		mCacheUpTo;	// max size
	uint32		mCacheSize; // current size
	double		**mCache;
};

#endif // __UNIFORMINTERESTINGNESS_H
