#ifndef __SEARCH_H
#define __SEARCH_H

#include <Primitives.h>
#include <Bass.h>
#include <collections/PatternCollection.h>

class Config;
class Database;
class Interestingness;
class Timer;

class Search {
public:
	Search(Config *config, Database *database, PatternCollection *patternCollection);
	virtual ~Search();

	// -- Virtual --
	// Returns number of candidate patterns considered
	virtual uint64				DoSearch() =0;
	virtual void				LogParameters();
	virtual string				GetName() =0;
	virtual bool				CanUseUpperBound()				{ return false; }

	// -- Get/set --
	uint32						GetElapsedTime(); // in seconds
	virtual uint32				GetNumRunsPerIteration() {return 1;} // mostly for Hillclimber
	Interestingness				*GetInterestingness()			{ return mInterestingness; }
	
	// -- Static --
	static Search* Create(Config *config, Database *db, PatternCollection *pCollection);
	static Search* Create(const string &desc, Config *config, Database *db, PatternCollection *pCollection);

protected:
	ItemSet			*ExtendPattern(ItemSet *pattern, const uint32 addNode, const uint32 newNumNodes);
	ItemSet			*ShrinkPattern(ItemSet *pattern, const uint32 removeNode, const uint32 newNumNodes);

	void			InitTimer();		// starts timer
	bool			OutOfTime();

	// The important variables
	Config		*mConfig;
	Database	*mDatabase;
	PatternCollection	*mPatternCollection;
	Interestingness *mInterestingness;

	// Generic search parameters
	double		mMinInterestingness;
	uint32		mMinNumNodes;
	uint32		mMaxSize;
	bool		mDisableUpperBound;
	bool		mUseUpperBound;

	// Stats
	uint64		mCandidateCount; 
	Timer		*mTimer;
	time_t		mMaxTime;
};

#endif // __SEARCH_H
